<?php

class rdc_store extends hook
{
	public $hookName = "rdc_store";
	public $active = true;			//enable/disable hook (true => enabled)
	public $timer = 0;
	public $luminance_min = 60;
	public $luminance_max = 3000;
	public $storeDownTime = 38;
	public $storeUpTime = 41;
	public $maxLevel;
	public $storeLevel;
	protected $propertyInitialized =array();
	protected $devicelist = array(
		RDC_EXTERIEUR_LUMINOSITE => "illuminance_lux",
		RDC_SALON_LUMINOSITE => "illuminance_lux",
		METEO => "rainin",
		METEO => "solarradiation",
		METEO => "windspeedkmh",
		METEO => "windgustkmh"
	);

	// callback fonction. Is called with these 3 parameters
	// $device 		-> calling device
	// $property 	-> property of the device (given by mqtt)
	// $value 		-> value of the property
	public function callBack($device, $property, $value)
	{
		global $mohaDB, $devices, $indexDevices;

		logger(DEBUG, "Callback : RDC_STORE", __FILE__ . ":" . __LINE__);
		$rain = 0;
		static $rainS;
		$exterieur_lux = 0;
		$exterieurLuxMoyen =0;
		$salon_lux = 0;
		$rafale = 0;
		$soleil = 0;
		$storeDevice = $indexDevices[RDC_STORE];
		//$this->maxLevel = 100;
		$r = 100;
		$moving = "STOP";
		$motorReversal = false;
		$store2level = false;

		if (array_key_exists("moving", $storeDevice->properties))
		{
			$moving = $storeDevice->properties["moving"]["value"];
			logger(DEBUG, "Moving = " . $moving, __FILE__ . ":" . __LINE__);
		}
		if (array_key_exists("motor_reversal", $storeDevice->properties))
		{
			if ($storeDevice->properties["motor_reversal"]["value"] == "ON")
			{
				$motorReversal = true;
			}else
			{
				$motorReversal = false;
			}
			logger(DEBUG, "Motor reversal =" . bool2string($motorReversal), __FILE__ . ":" . __LINE__);
		}

		if ($moving == "STOP")
		{
			if (array_key_exists("position", $storeDevice->properties))
			{
				if ($motorReversal)
				{
					$this->storeLevel = $storeDevice->properties["position"]["value"];
				}else
				{
					$this->storeLevel = 100 - $storeDevice->properties["position"]["value"];
				}
				logger(DEBUG, "storeLevel=" . $this->storeLevel, __FILE__ . ":" . __LINE__);
			}else
			{
				$this->storeLevel = 0;
				//return true;
			}

			if (array_key_exists(METEO, $indexDevices))
			{
				if (array_key_exists("rainin", $indexDevices[METEO]->properties))
				{
					$rainTmp = $indexDevices[METEO]->properties["rainin"]["value"];
					$rain = $rainTmp - $rainS;
					$rainS = $rainTmp;
					logger(DEBUG, "rain=" . $rain, __FILE__ . ":" . __LINE__);
				}
				if (array_key_exists("windgustkmh", $indexDevices[METEO]->properties))
				{
					if ($indexDevices[METEO]->properties["windgustkmh"]["value"] > 50)
					{
						logger(ALERT, "Vent fort :" . $indexDevices[METEO]->properties["windgustkmh"]["value"], __FILE__ . ":" . __LINE__);
					}
					$rafale = $mohaDB->moyenne($indexDevices[METEO], "windgustkmh", 6);
					if ($rafale == 0)
					{
						$rafale = $indexDevices[METEO]->properties["windgustkmh"]["value"];
					}
					logger(DEBUG, "rafale=" . $rafale, __FILE__ . ":" . __LINE__);
				}
				if (array_key_exists("windspeedkmh", $indexDevices[METEO]->properties))
				{
					if ($indexDevices[METEO]->properties["windspeedkmh"]["value"] > 50)
					{
						logger(ALERT, "Vent fort :" . $indexDevices[METEO]->properties["windspeedkmh"]["value"], __FILE__ . ":" . __LINE__);
					}
					$vent = $mohaDB->moyenne($indexDevices[METEO], "windspeedkmh", 6);
					if ($vent == 0)
					{
						$vent = $indexDevices[METEO]->properties["windspeedkmh"]["value"];
					}
					logger(DEBUG, "vitesse du vent =" . $vent, __FILE__ . ":" . __LINE__);
				}
				if ($vent > $rafale)
				{
					$rafale = $vent;
				}
				if (array_key_exists("solarradiation", $indexDevices[METEO]->properties))
				{
					$soleil = $mohaDB->moyenne($indexDevices[METEO], "solarradiation", 6);
					logger(DEBUG, "soleil=" . $soleil, __FILE__ . ":" . __LINE__);
				}
			}

			if (array_key_exists("illuminance_lux", $indexDevices[RDC_EXTERIEUR_LUMINOSITE]->properties))
			{
				$exterieur_lux = $indexDevices[RDC_EXTERIEUR_LUMINOSITE]->properties["illuminance_lux"]["value"];
				$exterieurLuxMoyen = $mohaDB->moyenne($indexDevices[RDC_EXTERIEUR_LUMINOSITE], "illuminance_lux", 5);
				if ($exterieurLuxMoyen == 0)
				{
					$exterieurLuxMoyen = $exterieur_lux;
				}
				logger(DEBUG, "exterieur_lux=" . $exterieur_lux . " et exterieurLuxMoyen=" . $exterieurLuxMoyen, __FILE__ . ":" . __LINE__);
			}
			if (array_key_exists("illuminance_lux", $indexDevices[RDC_SALON_LUMINOSITE]->properties))
			{
				$salon_lux = $mohaDB->moyenne($indexDevices[RDC_SALON_LUMINOSITE], "illuminance_lux", 10);
				if ($salon_lux == 0)
				{
					$salon_lux = $indexDevices[RDC_SALON_LUMINOSITE]->properties["illuminance_lux"]["value"];
				}
				logger(DEBUG, "salon_lux=" . $salon_lux, __FILE__ . ":" . __LINE__);
			}
			if ($exterieur_lux > 33000 or $salon_lux >600)
			{
				logger(DEBUG, "exterieur_lux > 33000 or salon_lux >600", __FILE__ . ":" . __LINE__);
				if ($this->maxLevel != 0 and $salon_lux > 100)
				{
					$store2level = $this->storeLevel + 15;
				}
			}elseif ($exterieurLuxMoyen < 6000 and $salon_lux < 200 and $soleil < 100)
			{
				logger(DEBUG, "exterieurLuxMoyen < 6000 and salon_lux < 200 and soleil < 100", __FILE__ . ":" . __LINE__);
				$this->close("Luminosité faible");
			}

			logger(DEBUG, sprintf(_("property=%s, value=%s"), $property, $value), __FILE__ . ":" . __LINE__);
			if ($rafale >= 70 or $rain != 0)
			{
				$this->maxLevel = 0;
				logger(DEBUG, sprintf("rafale = %s  and rain = %s",$rafale, $rain), __FILE__ . ":" . __LINE__);
				$this->close("vent trop fort ou pluie");
				return 0;
			}elseif ($rafale != 0)
			{
				//calcul du maxlevel par rapport a la vitesse des rafales
				$r = round(($rafale-20)/4, 0, PHP_ROUND_HALF_UP)*10;
				if ($r < 0) 	 $r = 0;
				if ($motorReversal)
				{
					$this->maxLevel = $r;
				}else
				{
					$this->maxLevel = 100 - $r;
				}
				logger(DEBUG, "rafale=" . $rafale, __FILE__ . ":" . __LINE__);
			}
			logger(DEBUG, "maxlevel=" . $this->maxLevel, __FILE__ . ":" . __LINE__);

			switch ($property)
			{
				case "rainin":
					logger(DEBUG, "CASE: rainin:" . $value, __FILE__ . ":" . __LINE__);
					$this->close("Pluie");
					break;
				case "windgustkmh";
					logger(DEBUG, "CASE: windgustkmh:" . $rafale, __FILE__ . ":" . __LINE__);
					break;
				default:
			}
			logger(DEBUG, "maxLevel = " . $this->maxLevel . ' result => $r = ' . $r, __FILE__ . ":" . __LINE__);
			if ($this->storeLevel > $this->maxLevel)
			{
				$this->setTo($this->maxLevel, AUTO);
			}elseif ($store2level !== false)		//$store2level = false then no change on store position
			{
				if ( $store2level > $this->maxLevel )
				{
					logger(DEBUG, "store2level > maxLevel", __FILE__ . ":" . __LINE__);
					$this->setTo($this->maxLevel, AUTO);
				}else
				{
					logger(DEBUG, "store2level <= maxLevel", __FILE__ . ":" . __LINE__);
					$this->setTo($store2level, AUTO);
				}
			}
		}else
		{
			logger(DEBUG, _("Store is Moving  ") . $moving, __FILE__ . ":" . __LINE__);
		}

	}

	private function setTo($level, $method)
	{
		logger(DEBUG, "function 'set' to level : " . $level, __FILE__ . ":" . __LINE__);
		if ($level <= 100)
		{
			logger(DEBUG, "Open store :" . $level, __FILE__ . ":" . __LINE__);
			$this->send(100 - $level, $method);
		}//else
		/*{
			logger(DEBUG, "store is already at level" . $this->storeLevel . " so < at " . $level, __FILE__ . ":" . __LINE__);
		}*/

	}

	private function close ($reason)
	{
		if ($this->storeLevel > 0)
		{
			logger(DEBUG, "Close store :" . $reason, __FILE__ . ":" . __LINE__);
			$this->send(100, IDLE);
		}
	}

	private function send($level, $method)
	{
		global $indexDevices;
		$deviceObject = $indexDevices[RDC_STORE];
		$msg = array("position" => $level);
		logger(INFO, sprintf(_("publishing message: %s to %s"), json_encode($msg), $deviceObject->friendlyName), __FILE__ . ":" . __LINE__);
		$deviceObject->payload = $msg;
		$deviceObject->set("position", $method);
	}
}

$hooks["rdc_store"] = new rdc_store();

?>
print_r($function[0], true)
